package org.nakedobjects.nos.remote.command;

import junit.framework.TestCase;

import org.easymock.MockControl;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.noa.util.ByteEncoder;
import org.nakedobjects.nof.reflect.remote.NakedObjectsRemoteException;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nof.reflect.remote.data.DummyIdentityData;
import org.nakedobjects.nof.reflect.remote.data.IdentityData;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;
import org.nakedobjects.testing.DummyOid;
import org.nakedobjects.testing.NullSession;


public class CommandClientTest extends TestCase {
    private MockControl control;
    private Distribution distribution;
    private CommandClient client;
    private NullSession session;

    protected void setUp() throws Exception {
        control = MockControl.createControl(Distribution.class);
        distribution = (Distribution) control.getMock();
        client = new CommandClient();
        client.setConnection(new CommandClientConnection() {
            public Response executeRemotely(Request request) {
                request.execute(distribution);
                return new Response(request);
            }

            public void init() {}

            public void shutdown() {}
        });
        session = new NullSession();
    }

    public void testOidForService() {
        distribution.oidForService(session, "domain.Service");
        Object data = new DummyIdentityData();
        control.setReturnValue(data);

        control.replay();
        IdentityData ret = client.oidForService(session, "domain.Service");
        control.verify();

        assertEquals(data, ret);
    }
    

    public void testHasInstances() {
        distribution.hasInstances(session, "pkg.Class");
        boolean data = true;
        control.setReturnValue(data);

        control.replay();
        boolean ret = client.hasInstances(session, "pkg.Class");
        control.verify();

        assertEquals(data, ret);
    }

    public void testOutOfSequence() {
        client.setConnection(new CommandClientConnection() {
            public Response executeRemotely(Request request) {
                // create a response based on another request so id is different
                return new Response(new AbstractRequest((Session) null) {
                    public void execute(Distribution distribution) {}

                    protected void doEncode(final ByteEncoder encoder) {}
               });
            }

            public void init() {}

            public void shutdown() {}
        });

        try {
            client.oidForService(session, "domain.Service");
            fail();
        } catch (NakedObjectsRemoteException e) {
            assertTrue(e.getMessage().startsWith("Response out of sequence"));
        }
    }

    public void testClearAssociation() {
        DummyIdentityData target = new DummyIdentityData(new DummyOid(1), "class 1", null);
        DummyIdentityData associate = new DummyIdentityData(new DummyOid(2), "class 2", null);
        distribution.clearAssociation(session, "fieldname", target, associate);
        Object data = new ObjectData[2];
        control.setReturnValue(data);

        control.replay();
        ObjectData[] ret = client.clearAssociation(session, "fieldname", target, associate);
        control.verify();

        assertEquals(data, ret);
    }

}

// Copyright (c) Naked Objects Group Ltd.
